unit ChildEventUnit;

interface

uses
  SysUtils, Classes, ChildMainFormUnit;

type
  TTestChildEvent = class(TThread)
  private
    FUIString: String;
    //Two routines to simplify showing our progress
    procedure UpdateUI;
    procedure WriteString(const S: String);
  protected
    //Body of the thread
    procedure Execute; override;
    //This relies on the event having happened
    procedure CodeReliantOnTheEvent;
  public
    constructor Create;
    destructor Destroy; override;
  end;

implementation

uses
  Windows;

const
  EventName = 'SampleEvent';

constructor TTestChildEvent.Create;
begin
  inherited Create(False);
  WriteString(Format('Thread %x started', [ThreadID]))
end;

destructor TTestChildEvent.Destroy;
begin
  WriteString(Format('Thread %x terminated', [ThreadID]));
  inherited Destroy
end;

procedure TTestChildEvent.UpdateUI;
begin
  Form1.Listbox1.Items.Add(FUIString)
end;

procedure TTestChildEvent.WriteString(const S: String);
begin
  FUIString := S;
  Synchronize(UpdateUI);
end;

procedure TTestChildEvent.Execute;
var
  LocalEvent: THandle;
begin
  LocalEvent := OpenEvent(Event_All_Access, False, EventName);
  if LocalEvent = 0 then
  begin
    WriteString('event not found');
    Exit
  end;
  WriteString(Format(
    'Thread %x waiting for event', [ThreadID]));
  WaitForSingleObject(LocalEvent, Infinite);
  CodeReliantOnTheEvent;
  CloseHandle(LocalEvent);
end;

procedure TTestChildEvent.CodeReliantOnTheEvent;
begin
  WriteString(Format(
    'Thread %x has seen the event', [ThreadID]));
end;

end.