unit ChildMutexUnit;

interface

uses
  SysUtils, Classes, ChildMainFormUnit;

type
  TTestChildMutex = class(TThread)
  private
    FUIString: String;
    //Two routines to simplify showing our progress
    procedure UpdateUI;
    procedure WriteString(const S: String);
  protected
    //Body of the thread
    procedure Execute; override;
    //This relies on mutually exclusive access
    procedure ThisNeedsMutualExclusiveAccess;
  public
    constructor Create;
    destructor Destroy; override;
  end;

implementation

uses
  Windows;

const
  MutexName = 'SampleMutex';

constructor TTestChildMutex.Create;
begin
  inherited Create(False);
  WriteString(Format('Thread %x started', [ThreadID]))
end;

destructor TTestChildMutex.Destroy;
begin
  WriteString(Format('Thread %x terminated', [ThreadID]));
  inherited Destroy
end;

procedure TTestChildMutex.UpdateUI;
begin
  Form1.Listbox1.Items.Add(FUIString)
end;

procedure TTestChildMutex.WriteString(const S: String);
begin
  FUIString := S;
  Synchronize(UpdateUI);
end;

procedure TTestChildMutex.Execute;
var
  LocalMutex: THandle;
begin
  LocalMutex := OpenMutex(Mutex_All_Access, False, MutexName);
  if LocalMutex = 0 then
  begin
    WriteString('Mutex not found');
    Exit
  end;
  WriteString(Format(
    'Thread %x waiting for mutex', [ThreadID]));
  WaitForSingleObject(LocalMutex, Infinite);
  ThisNeedsMutualExclusiveAccess;
  ReleaseMutex(LocalMutex);
  CloseHandle(LocalMutex);
end;

procedure TTestChildMutex.ThisNeedsMutualExclusiveAccess;
begin
  WriteString(Format(
    'Thread %x got mutex', [ThreadID]));
  Sleep(4000);
  WriteString(Format(
    'Thread %x finished with mutex', [ThreadID]));
end;

end.