unit ParentSemaphoreUnit;

interface

uses
  Classes, ParentMainFormUnit, Windows;

type
  TTestParentSemaphore = class(TThread)
  private
    FUIString: String;
    //Two routines to simplify showing our progress
    procedure UpdateUI;
    procedure WriteString(const S: String);
  protected
    //Body of the thread
    procedure Execute; override;
    //Can only execute 3 simultaneous times
    procedure Max3SimultaneousExecutions;
  public
    constructor Create;
    destructor Destroy; override;
  end;

implementation

uses
  SysUtils;

var
  Semaphore: THandle;

const
  SemaphoreName = 'SampleSemaphore';
  MaxSemaphoreCount = 3;

constructor TTestParentSemaphore.Create;
begin
  inherited Create(False);
  WriteString(Format('Thread %x started', [ThreadID]))
end;

destructor TTestParentSemaphore.Destroy;
begin
  WriteString(Format('Thread %x terminated', [ThreadID]));
  inherited Destroy
end;

procedure TTestParentSemaphore.UpdateUI;
begin
  Form1.Listbox1.Items.Add(FUIString)
end;

procedure TTestParentSemaphore.WriteString(const S: String);
begin
  FUIString := S;
  Synchronize(UpdateUI);
end;

procedure TTestParentSemaphore.Execute;
const
  //Not defined in Windows.Pas
  Semaphore_All_Access =
    Standard_Rights_Required or Windows.Synchronize or 3;
var
  SemaphoreCount: Integer;
begin
  WriteString(Format(
    'Thread %x waiting for Semaphore', [ThreadID]));
  WaitForSingleObject(Semaphore, Infinite);
  Max3SimultaneousExecutions;
  ReleaseSemaphore(Semaphore, 1, @SemaphoreCount);
  WriteString(Format(
    'Thread %x released semaphore whose count was %d',
    [ThreadID, SemaphoreCount]));
end;

procedure TTestParentSemaphore.Max3SimultaneousExecutions;
begin
  WriteString(Format(
    'Thread %x got Semaphore', [ThreadID]));
  Sleep(4000);
  WriteString(Format(
    'Thread %x finished with Semaphore', [ThreadID]));
end;

initialization
  Semaphore := CreateSemaphore(
    nil, MaxSemaphoreCount, MaxSemaphoreCount, SemaphoreName)
finalization
  //This is not strictly necessary as all open
  //ThreadIDs are closed by the Win32 OS upon termination
  if Semaphore <> 0 then
    CloseHandle(Semaphore)
end.
