unit RunAppU;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, Menus, ShellAPI, ToolHelp;

type
  { Used by FindModule and ModuleRunning }
  PProgID = ^TProgID;
  TProgID = record
    Wnd: HWnd;
    Instance, Task: THandle;
  end;

  TForm1 = class(TForm)
    AppEdt: TEdit;
    Label1: TLabel;
    Label2: TLabel;
    ParamsEdt: TEdit;
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    Select1: TMenuItem;
    Run1: TMenuItem;
    WinExec1: TMenuItem;
    LoadModule1: TMenuItem;
    ShellExecute1: TMenuItem;
    LoadModuleBtn: TButton;
    WinExecBtn: TButton;
    ShellExecBtn: TButton;
    OpenDialog1: TOpenDialog;
    Label3: TLabel;
    Label4: TLabel;
    ModNameLbl: TLabel;
    FileNameLbl: TLabel;
    procedure Select1Click(Sender: TObject);
    procedure LoadModuleClick(Sender: TObject);
    procedure WinExecClick(Sender: TObject);
    procedure ShellExecClick(Sender: TObject);
    procedure AppEdtChange(Sender: TObject);
  private
    { Private declarations }
    Buf1, Buf2: array[0..255] of Char;
    Environment: PChar;
    OldSEM: Word;
    Instance: THandle;
  public
    { Public declarations }
    function CheckExec(Instance: THandle): THandle;
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

function FindModuleProc(Window: HWnd; UserData: PProgID): Bool; export;
begin
  Result := True;
  with UserData^ do
    if GetWindowWord(Window, gww_HInstance) = Instance then
    begin
      Wnd := Window;
      Task := GetWindowTask(Wnd);
      Result := False;
    end;
end;

{ Locate an application, if possible, by the instance }
{ handle set up in the TProgID record parameter }
function FindModule(var ProgID: TProgID): Boolean;
begin
  Result := ProgID.Instance >= hInstance_Error;
  if not Result then
    Exit;
  EnumWindows(@FindModuleProc, Longint(@ProgID));
  Result := ProgID.Wnd <> 0;
end;

{ Identify if the application is still running using }
{ the fields of the TProgID record parameter }
{$B-}
function ModuleRunning(ProgID: TProgID): Boolean;
begin
  with ProgID do
    Result := IsWindow(Wnd) and IsTask(Task) and
              (GetWindowWord(Wnd, gww_HInstance) = Instance) and
              (GetWindowTask(Wnd) = Task);
end;

procedure TForm1.Select1Click(Sender: TObject);
begin
  with OpenDialog1 do
    if Execute then
      AppEdt.Text := FileName;
end;

function FillEnvironment(Env: PChar): PChar;
const
  EnvironStrs: array[1..3] of PChar =
    ('STARTDIR=C:\DOS', 'BACKUP=YES', 'NET=NOVELL');
var
  Loop: Byte;
  PtrEnv: PChar;
begin
  Result := Env;
  PtrEnv := Env;
  for Loop := 1 to 3 do
  begin
    StrCopy(PtrEnv, EnvironStrs[Loop]);
    Inc(PtrEnv, Succ(StrLen(PtrEnv)));
  end;
  StrCopy(PtrEnv, '');
end;

const
  MaxLoadModuleCmdLine = 120; { Max length of command-line used by LoadModule }

type
  { These two structures are for use in }
  { conjunction with the LoadModule API }
  PCmdShowRec = ^TCmdShowRec;
  TCmdShowRec = packed record
    Two: Word;
    CmdShow: Word;
  end; { TCmdShowRec }

  TLoadParms = packed record
    EnvSeg: Cardinal;
    CmdLine: PChar;
    CmdShowPtr: PCmdShowRec;
    Reserved: Longint;
  end; { TLoadParms }

{ LoadModule is rather clumsy, but you can
  use it if you need a custom DOS environment }
procedure TForm1.LoadModuleClick(Sender: TObject);
const
  CmdShowRec: TCmdShowRec = (Two: 2; CmdShow: sw_ShowNormal);
  LoadParms: TLoadParms =
    (EnvSeg: 0; CmdLine: nil; CmdShowPtr: @CmdShowRec; Reserved: 0);
  EnvSize = 1024;
var
  ProgID: TProgID;
begin
  StrPCopy(Buf1, AppEdt.Text);
  StrPCopy(Buf2, Copy(#32 + ParamsEdt.Text, 1, MaxLoadModuleCmdLine));
  LoadParms.CmdLine := Buf2;
  GetMem(Environment, EnvSize);
  LoadParms.EnvSeg := HiWord(Longint(FillEnvironment(Environment)));
  { Stop the system-modal error box if the file is not found }
  OldSEM := SetErrorMode(sem_NoOpenFileErrorBox);
  Instance := CheckExec(LoadModule(Buf1, @LoadParms));
  FreeMem(Environment, EnvSize);
  ProgID.Instance := Instance;
  if FindModule(ProgID) then
    while ModuleRunning(ProgID) and not Application.Terminated do
      Application.ProcessMessages;
  if not ModuleRunning(ProgID) then
    ShowMessage('It''s finished!');
  { Set error mode back to what it was }
  SetErrorMode(OldSEM);
end;

procedure TForm1.WinExecClick(Sender: TObject);
var
  ProgID: TProgID;
begin
  FillChar(ProgID, SizeOf(ProgID), #0);
  StrPCopy(Buf1, AppEdt.Text + #32 + ParamsEdt.Text);
  OldSEM := SetErrorMode(sem_NoOpenFileErrorBox);
  Instance := CheckExec(WinExec(Buf1, sw_ShowNormal));
  ProgID.Instance := Instance;
  if FindModule(ProgID) then
    while ModuleRunning(ProgID) and not Application.Terminated do
      Application.ProcessMessages;
  if not ModuleRunning(ProgID) then
    ShowMessage('It''s finished!');
  SetErrorMode(OldSEM);
end;

procedure TForm1.ShellExecClick(Sender: TObject);
var
  ProgID: TProgID;
begin
  StrPCopy(Buf1, AppEdt.Text);
  StrPCopy(Buf2, ParamsEdt.Text);
  Instance := CheckExec(
    ShellExecute(Handle, nil, Buf1, Buf2, nil, sw_ShowNormal));
  ProgID.Instance := Instance;
  if FindModule(ProgID) then
    while ModuleRunning(ProgID) and not Application.Terminated do
      Application.ProcessMessages;
  if not ModuleRunning(ProgID) then
    ShowMessage('It''s finished!');
end;

procedure TForm1.AppEdtChange(Sender: TObject);
begin
  Run1.Enabled := AppEdt.Text <> '';
  LoadModuleBtn.Enabled := Run1.Enabled;
  WinExecBtn.Enabled := Run1.Enabled;
  ShellExecBtn.Enabled := Run1.Enabled;
end;

function TForm1.CheckExec(Instance: THandle): THandle;
var
  ModuleEntry: TModuleEntry;
begin
  ModNameLbl.Caption := '?';
  FileNameLbl.Caption := '?';
  Result := Instance;
  if Instance < hInstance_Error then
    MessageDlg('Failed to execute. Error ' + IntToStr(Instance),
      mtError, [mbOk], 0)
  else
  begin
    ModuleEntry.dwSize := SizeOf(TModuleEntry);
    if ModuleFindHandle(@ModuleEntry, GetInstanceModule(Instance)) = 0 then
      Exit;
    ModNameLbl.Caption := StrPas(ModuleEntry.szModule);
    FileNameLbl.Caption := StrPas(ModuleEntry.szExePath);
  end;
end;

end.
